/*
 *  OpenVPN-GUI -- A Windows GUI for OpenVPN.
 *
 *  Copyright (C) 2004 Mathias Sundman <mathias@nilings.se>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program (see the file COPYING included with this
 *  distribution); if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <windows.h>
#include "tray.h"
#include "openvpn.h"

/* bool definitions */
#define bool int
#define true 1
#define false 0


//POPUP MENU
HMENU hMenu;		// menu handle
HMENU hMenuConnect;	// menu handle
HMENU hMenuDisconnect;	// menu handle
HMENU hMenuViewLog;	// menu handle
HWND  hwnd;     	// main window handle

NOTIFYICONDATA ni;


// Mouse clicks on tray
void OnNotifyTray(LPARAM lParam)
{
  POINT pt;				// point structure

  // Right click, show the menu
  switch(lParam) {
    case WM_RBUTTONDOWN: 
      GetCursorPos(&pt);		// get the cursors position
      SetForegroundWindow(hwnd);	// set the foreground window
      TrackPopupMenu(hMenu,TPM_RIGHTALIGN,pt.x,pt.y,0,hwnd,NULL);// track the popup
      break;
    case WM_LBUTTONDOWN:
      break; 
  }
  PostMessage(hwnd,WM_NULL,0,0);// post a null message
}


void OnDestroyTray()
{
  //Destroy popup menu
  DestroyMenu(hMenu);
	
  //Remove Icon from Tray
  Shell_NotifyIcon(NIM_DELETE, &ni);
}


//Initialize tray
void TRAY_Init(HWND hWnd)
{
    hwnd = hWnd;
    hMenu=CreatePopupMenu();
    hMenuConnect=CreatePopupMenu();
    hMenuDisconnect=CreatePopupMenu();
    hMenuViewLog=CreatePopupMenu();
    CreateItemList(hMenu); 
    LoadAppIcon();
} 


//Create menu items
void CreateItemList(HMENU hMenu)
{
  extern char config_files[MAX_CONFIGS][MAX_PATH];
  extern char ext_string[];
  extern int num_configs;
  char conn_name[200];
  int i;
  
  // CREATE POPUP MENU ( on the fly )
  AppendMenu(hMenu,MF_POPUP,(UINT) hMenuConnect,IDM_TEXT_CONNECT); 
  AppendMenu(hMenu,MF_POPUP,(UINT) hMenuDisconnect,IDM_TEXT_DISCONNECT);
  AppendMenu(hMenu,MF_SEPARATOR,0,0);
  AppendMenu(hMenu,MF_POPUP,(UINT) hMenuViewLog,IDM_TEXT_VIEWLOG);
  AppendMenu(hMenu,MF_SEPARATOR,0,0);
  AppendMenu(hMenu,MF_STRING ,IDM_CLOSE,IDM_TEXT_CLOSE);

  /* Create Connect/Disconnect POPUP MENU */
  for (i=0; i < num_configs; i++) {

    /* Cut of extention from config filename. */
    strncpy(conn_name, config_files[i], sizeof(conn_name));
    conn_name[strlen(conn_name) - (strlen(ext_string)+1)]=0;

    AppendMenu(hMenuConnect,MF_STRING, IDM_CONNECTMENU+i, conn_name);  
    AppendMenu(hMenuDisconnect,MF_STRING, IDM_DISCONNECTMENU+i, conn_name);  
    AppendMenu(hMenuViewLog,MF_STRING, IDM_VIEWLOGMENU+i, conn_name);  
    SetMenuStatus(i, false); 
  }
}


BOOL LoadAppIcon()
{
  // Load icon from resource
  HICON hIcon = (HICON)LoadImage(GetModuleHandle(NULL), MAKEINTRESOURCE(APP_ICON), 
                                                IMAGE_ICON, 0, 0, LR_DEFAULTCOLOR);
  if (hIcon) {
    SendMessage(hwnd, WM_SETICON, (WPARAM) (ICON_SMALL), (LPARAM) (hIcon));
    SendMessage(hwnd, WM_SETICON, (WPARAM) (ICON_BIG), (LPARAM) (hIcon));  //ALT+TAB icon
    return TRUE;
  }
  return FALSE;
}


void ShowTrayIcon()
{
  ni.cbSize = sizeof(ni);
  ni.uID = 0;
  lstrcpyn(ni.szTip, MSG_TIP, sizeof(ni.szTip)); 
  ni.hWnd = hwnd;
  ni.uFlags = NIF_MESSAGE | NIF_TIP | NIF_ICON; // We want to use icon, tip, and callback message
  ni.uCallbackMessage = WM_NOTIFYICONTRAY;      // Our custom callback message (WM_APP + 1)
    
     
  //Load selected icon
  ni.hIcon = (HICON)LoadIcon((HINSTANCE)GetWindowLong(hwnd, GWL_HINSTANCE), 
                                                       MAKEINTRESOURCE(APP_ICON));	

  Shell_NotifyIcon(NIM_ADD, &ni);       
}

void SetMenuStatus (int config, BOOL bCheck)
{
  int iState ;
  iState = (bCheck) ? MF_CHECKED : MF_UNCHECKED ;
  CheckMenuItem (hMenuConnect, IDM_CONNECTMENU + config, iState) ;
  CheckMenuItem (hMenuDisconnect, IDM_DISCONNECTMENU + config, iState) ;
  
  if (bCheck) {
    EnableMenuItem(hMenuConnect, IDM_CONNECTMENU + config, MF_GRAYED);
    EnableMenuItem(hMenuDisconnect, IDM_DISCONNECTMENU + config, MF_ENABLED);
  }
  else {
    EnableMenuItem(hMenuConnect, IDM_CONNECTMENU + config, MF_ENABLED);
    EnableMenuItem(hMenuDisconnect, IDM_DISCONNECTMENU + config, MF_GRAYED);
  }
}
